@@ -1,4 +1,3 @@
-import json
 import threading
 import tornado.ioloop
 import tornado.httpserver
@@ -9,6 +8,13 @@ from testify import assert_equal, setup_teardown, TestCase
 from testify.test_runner import TestRunner
 from testify.plugins.http_reporter import HTTPReporter
 
+try:
+    import simplejson as json
+    _hush_pyflakes = [json]
+    del _hush_pyflakes
+except ImportError:
+    import json
+
 
 class DummyTestCase(TestCase):
     __test__ = False
@@ -39,7 +45,7 @@ class HTTPReporterTestCase(TestCase):
         app = tornado.web.Application([(r"/results", ResultsHandler)])
         srv = tornado.httpserver.HTTPServer(app)
         srv.listen(0)
-        portnum = srv._socket.getsockname()[1]
+        portnum = self.get_port_number(srv)
 
         iol = tornado.ioloop.IOLoop.instance()
         thread = threading.Thread(target=iol.start)
@@ -53,6 +59,13 @@ class HTTPReporterTestCase(TestCase):
         iol.stop()
         thread.join()
 
+    def get_port_number(self, server):
+        if hasattr(server, "_sockets"): # tornado > 2.0
+            _socket = server._sockets.values()[0]
+        else: # tornado 1.2 or earlier
+            _socket = server._socket
+        return _socket.getsockname()[1]
+
     def test_http_reporter_reports(self):
         """A simple test to make sure the HTTPReporter actually reports things."""
 
@@ -76,4 +89,3 @@ class HTTPReporterTestCase(TestCase):
         assert_equal(first['runner_id'], 'tries_twice')
         assert_equal(first, second)
 
-# vim: set ts=4 sts=4 sw=4 et:
