@@ -13,8 +13,6 @@
 # limitations under the License.
 from __future__ import with_statement
 
-import datetime as _datetime
-import functools
 import re
 
 import contextlib
@@ -493,33 +491,3 @@ def assert_exactly_one(*args, **kwargs):
         raise AssertionError("Expected exactly one True (got %d) args: %r" % (len(true_args), args))
 
     return true_args[0]
-
-
-def assert_takes_less_than(max_time_in_ms):
-    """A decorator, primarily for test methods, that asserts that the method completes
-    within a certain amount of time.
-
-    Example:
-
-        @assert_takes_less_than(2*1000)
-        def foo(self):
-            # Do stuff
-
-        This would assert that foo takes less than 2 seconds to run.
-    """
-    def decorator(method):
-        @functools.wraps(method)
-        def method_wrapper(*args, **kwargs):
-            start_time = _datetime.datetime.now()
-
-            result = method(*args, **kwargs)
-
-            delta = _datetime.datetime.now() - start_time
-            msecs = delta.seconds * 1000.0 + delta.microseconds / 1000.0
-
-            assert_lt(msecs, max_time_in_ms)
-            return result
-
-        return method_wrapper
-
-    return decorator
