@@ -20,11 +20,12 @@ __testify = 1
 from collections import defaultdict
 import functools
 import pprint
-
+import sys
 
 from test_case import MetaTestCase, TestCase
 import test_discovery
 
+
 class TestRunner(object):
     """TestRunner is the controller class of the testify suite.
 
@@ -97,7 +98,13 @@ class TestRunner(object):
                         )
                         yield test_case
 
-        discovered_tests = list(discover_inner())
+        discovered_tests = []
+        try:
+            discovered_tests = list(discover_inner())
+        except test_discovery.DiscoveryError, exc:
+            for reporter in self.test_reporters:
+                reporter.test_discovery_failure(exc)
+            sys.exit(1)
         test_case_count = len(discovered_tests)
         test_method_count = sum(len(list(test_case.runnable_test_methods())) for test_case in discovered_tests)
         for reporter in self.test_reporters:
@@ -179,3 +186,5 @@ class TestRunner(object):
 
         pp = pprint.PrettyPrinter(indent=2)
         print(pp.pformat([self.get_test_method_name(test) for test in test_list]))
+
+# vim: set ts=4 sts=4 sw=4 et:
